<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://furgonetka.pl
 * @since      1.0.0
 *
 * @package    Furgonetka
 * @subpackage Furgonetka/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two hooks
 *
 * @package    Furgonetka
 * @subpackage Furgonetka/admin
 * @author     Furgonetka.pl <woocommerce@furgonetka.pl>
 */
class Furgonetka_Admin
{
    const API_SOAP_URL      = 'https://biznes.furgonetka.pl/api/soap/v2?wsdl';
	const TEST_API_SOAP_URL = 'https://biznes.sandbox.furgonetka.pl/api/soap/v2?wsdl';

    const API_OAUTH_URL      = 'https://konto.furgonetka.pl/oauth';
	const TEST_API_OAUTH_URL = 'https://konto.sandbox.furgonetka.pl/oauth';

    const OPTION_CHECKOUT_UUID      = 'checkout_uuid';
    const OPTION_CHECKOUT_ACTIVE    = 'checkout_active';
    const OPTION_CHECKOUT_TEST_MODE = 'checkout_test_mode';

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * Error array.
     *
     * @since    1.0.0
     * @access   public
     * @var      array $errors Array of error messages.
     */
    public $errors;

    /**
     * Messages array.
     *
     * @since    1.0.0
     * @access   public
     * @var      array $messages Array of messages.
     */
    public $messages;

    /**
     * Furgonetka_admin_metaboxes class.
     *
     *  @var furgonetka_admin_metaboxes
     *
     */
    private $furgonetka_admin_metaboxes;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name The name of this plugin.
     * @param      string $version The version of this plugin.
     */
    public function __construct( $plugin_name, $version )
    {
        $this->plugin_name = $plugin_name;
        $this->version     = $version;

        $this->define_constans();

        require_once FURGONETKA_PLUGIN_DIR . 'includes/class-furgonetka-admin-metaboxes.php';
        $this->furgonetka_admin_metaboxes = new furgonetka_admin_metaboxes( $this );

        if ( empty( self::get_rest_customer_secret() ) || empty( self::get_rest_customer_secret() ) ) {
            $this->delete_credentials_data();
        }
    }

    /**
     * Define constans
     *
     * @return void
     */
    public function define_constans()
    {
        define( 'FURGONETKA_ADMIN_TEMPLATE_PARTS', plugin_dir_path( __FILE__ ) . 'admin/partials/template_parts' );
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since 1.0.0
     */
    public function enqueue_styles()
    {
        $file_path = 'css/furgonetka-admin.css';

        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . $file_path,
            array(),
            filemtime( plugin_dir_path( __FILE__ ) . $file_path ),
            'all'
        );
    }

    /**
     * Add relevant links to plugins page.
     *
     * @since 1.0.0
     *
     * @param array $links Plugin action links.
     *
     * @return array Plugin action links
     */
    public function plugin_action_links( $links )
    {
        $plugin_links = array();

        $plugin_links[] = '<a href="' . esc_url( get_admin_url( null, 'admin.php?page=' . $this->plugin_name ) ) . '">'
            . esc_html__( 'Settings', 'furgonetka' ) . '</a>';

        $plugin_links[] = '<a href="mailto:ecommerce@furgonetka.pl">' .
            esc_html__( 'Contact', 'furgonetka' ) . '</a>';

        return array_merge( $plugin_links, $links );
    }

    /**
     * Add furgonetka Page to woocommerce menu
     *
     * @since 1.0.0
     */
    public function furgonetka_menu()
    {
        add_submenu_page(
            'woocommerce',
            __( 'Furgonetka', 'furgonetka' ),
            __( 'Furgonetka', 'furgonetka' ),
            'manage_woocommerce',
            $this->plugin_name,
            array( $this, 'furgonetka_options' )
        );
    }

    /**
     * Return plugin name
     *
     * @return string
     */
    public function get_plugin_name(): string
    {
        return $this->plugin_name;
    }

    public static function get_checkout_uuid()
    {
        return self::get_plugin_option( self::OPTION_CHECKOUT_UUID );
    }

    public static function is_checkout_active()
    {
        return (bool) self::get_plugin_option( self::OPTION_CHECKOUT_ACTIVE );
    }

    public static function is_checkout_test_mode()
    {
        return (bool) self::get_plugin_option( self::OPTION_CHECKOUT_TEST_MODE );
    }

    private static function get_plugin_option( $option_name )
    {
        return get_option( FURGONETKA_PLUGIN_NAME . '_' . $option_name );
    }

    public function update_checkout_options( $uuid, $active, $test_mode )
    {
            update_option( $this->plugin_name . '_' . self::OPTION_CHECKOUT_UUID, sanitize_text_field( strval( $uuid ) ) );
            update_option( $this->plugin_name . '_' . self::OPTION_CHECKOUT_ACTIVE, (int) $active );
            update_option( $this->plugin_name . '_' . self::OPTION_CHECKOUT_TEST_MODE, (int) $test_mode );
    }

    /**
     * Furgonetka options page
     *
     * @since 1.0.0
     */
    public function furgonetka_options()
    {
        $reset = false;

        if ( ! current_user_can( 'manage_woocommerce' ) ) {
            wp_die( esc_html__( 'You do not have sufficient permissions to access this page.' ) );
        }

        $action = '';
        if (
            isset( $_POST['_wpnonce'], $_POST['furgonetkaAction'] )
            && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) )
        ) {
            $action = sanitize_text_field( wp_unslash( $_POST['furgonetkaAction'] ) );
        }

        if (
            ! isset( $_GET['action'] )
            || ( isset( $_GET['action'] )
            && sanitize_text_field( wp_unslash( $_GET['action'] ) ) !== 'oAuthComplete' )
        ) {
            if ( self::get_cient_id() && self::get_client_secret() && self::get_source_id() === false ) {
                $this->reset_credentials();
                $reset = true;
            }
        }

        if ( isset( $_GET['action'] ) && sanitize_text_field( wp_unslash( $_GET['action'] ) ) === 'oAuthComplete' ) {
            $this->save_credentials_code();
        }
        if ( isset( $action ) && 'save_sender_data' === $action ) {
            $this->save_sender_data();
        }
        if ( isset( $action ) && 'save_delivery' === $action ) {
            $this->save_delivery();
        }
        if ( isset( $action ) && 'reset_credentials' === $action ) {
            $this->reset_credentials();
            $reset = true;
        }
        if ( isset( $action ) && 'create_integration' === $action ) {
            $this->create_integration();
        }
        if ( isset( $action ) && 'save_advanced' === $action ) {
            $this->save_advanced_settings();
        }

        if ( get_option( $this->plugin_name . '_expires_date' ) > strtotime( 'now' ) ) {
            $access_token = get_option( $this->plugin_name . '_access_token' );

            if ( isset( $_GET['idOrder'] ) ) {
                try {
                    $url                         = self::get_package_form_url(
                        $access_token,
                        (int) sanitize_text_field( wp_unslash( $_GET['idOrder'] ) )
                    );
                    $furgonetka_form_url         = esc_url(
                        get_admin_url(
                            null,
                            'admin.php?page=' . $this->plugin_name
                        )
                    );
                    $furgonetka_errors           = $this->errors;
                    $furgonetka_messages         = $this->messages;
                    $furgonetka_package_from_url = $url;
                    // include admin view.
                    require_once 'partials/furgonetka-admin-getpackageform.php';
                    return;
                } catch ( Exception $e ) {
                    $this->errors[]      = $e->getMessage();
                    $furgonetka_errors   = $this->errors;
                    $furgonetka_messages = $this->messages;
                    require_once 'partials/furgonetka-admin-display.php';
                }
                return;
            } else {
                // view variables.
                $furgonetka_form_url = esc_url( get_admin_url( null, 'admin.php?page=' . $this->plugin_name ) );
                $furgonetka_errors   = $this->errors;
                $furgonetka_messages = $this->messages;
                // include admin view.
                require_once 'partials/furgonetka-admin-display.php';
            }
        } else {
            if ( ! $reset && get_option( $this->plugin_name . '_expires_date' ) ) {
                $this->reset_credentials();
            }

            // view variables.
            $furgonetka_form_url = esc_url( get_admin_url( null, 'admin.php?page=' . $this->plugin_name ) );
            $furgonetka_errors   = $this->errors;
            $furgonetka_messages = $this->messages;
            // include admin view.
            require_once 'partials/furgonetka-admin-display.php';
        }
    }

    /**
     * Validate user by code and save
     *
     * @since 1.0.0
     */
    private function save_credentials_code()
    {
        $code  = isset( $_GET['code'] ) ? urldecode( sanitize_text_field( wp_unslash( $_GET['code'] ) ) ) : null;
        $state = isset( $_GET['state'] ) ? urldecode( sanitize_text_field( wp_unslash( $_GET['state'] ) ) ) : null;

        if ( ! wp_verify_nonce( $state, 'furgonetka_csrf' ) ) {
            $this->errors[] = 'Incorrect CSRF';
            return;
        }

        $test                = self::get_test_mode();
        $key_consumer_key    = get_option( $this->plugin_name . '_key_consumer_key' );
        $key_consumer_secret = get_option( $this->plugin_name . '_key_consumer_secret' );

        try {
            $this->grant_code_access( $code, self::get_cient_id(), self::get_client_secret(), $test );

            $access_token = get_option( $this->plugin_name . '_access_token' );

            $integration_identifiers = $this->add_integration_source(
                $access_token,
                $key_consumer_key,
                $key_consumer_secret
            );
            $source_id               = $integration_identifiers->source_id ?? null;
            $integration_uuid        = $integration_identifiers->integration_uuid ?? null;

            if ( is_int( $source_id ) && is_string( $integration_uuid ) ) {
                update_option( $this->plugin_name . '_source_id', $source_id );
                update_option( $this->plugin_name . '_integration_uuid', $integration_uuid );
                $this->messages[] = __( 'Access granted', 'furgonetka' );
            }
        } catch ( Exception $e ) {
            $this->delete_credentials_data();
            $this->errors[] = $e->getMessage();
        }
    }

    /**
     * Delete credentials data.
     *
     * @return void
     */
    private function delete_credentials_data()
    {
        delete_option( $this->plugin_name . '_source_id' );
        delete_option( $this->plugin_name . '_integration_uuid' );
        delete_option( $this->plugin_name . '_access_token' );
        delete_option( $this->plugin_name . '_refresh_token' );
        delete_option( $this->plugin_name . '_expires_date' );

        delete_option( $this->plugin_name . '_key_consumer_key' );
        delete_option( $this->plugin_name . '_key_consumer_secret' );
    }

    /**
     * Save sender data
     *
     * @since 1.0.0
     */
    private function save_sender_data()
    {
        $name          = '';
        $company_name  = '';
        $post_code     = '';
        $city          = '';
        $street        = '';
        $email         = '';
        $telephone     = '';
        $iban          = '';
        $account_owner = '';

        if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) ) ) {
            $name          = isset( $_POST['name'] ) ?
                sanitize_text_field( wp_unslash( $_POST['name'] ) ) : '';
            $company_name  = isset( $_POST['companyName'] ) ?
                sanitize_text_field( wp_unslash( $_POST['companyName'] ) ) : '';
            $post_code     = isset( $_POST['postCode'] ) ?
                sanitize_text_field( wp_unslash( $_POST['postCode'] ) ) : '';
            $city          = isset( $_POST['city'] ) ?
                sanitize_text_field( wp_unslash( $_POST['city'] ) ) : '';
            $street        = isset( $_POST['street'] ) ?
                sanitize_text_field( wp_unslash( $_POST['street'] ) ) : '';
            $email         = isset( $_POST['email'] ) ?
                sanitize_email( wp_unslash( $_POST['email'] ) ) : '';
            $telephone     = isset( $_POST['telephone'] ) ?
                sanitize_text_field( wp_unslash( $_POST['telephone'] ) ) : '';
            $iban          = isset( $_POST['iban'] ) ?
                sanitize_text_field( wp_unslash( $_POST['iban'] ) ) : '';
            $account_owner = isset( $_POST['accountOwner'] ) ?
                sanitize_text_field( wp_unslash( $_POST['accountOwner'] ) ) : '';
        }

        update_option( $this->plugin_name . '_sender_name', $name );
        update_option( $this->plugin_name . '_sender_companyName', $company_name );
        update_option( $this->plugin_name . '_sender_postCode', $post_code );
        update_option( $this->plugin_name . '_sender_city', $city );
        update_option( $this->plugin_name . '_sender_street', $street );
        update_option( $this->plugin_name . '_sender_email', $email );
        update_option( $this->plugin_name . '_sender_telephone', $telephone );
        update_option( $this->plugin_name . '_cod_iban', $iban );
        update_option( $this->plugin_name . '_cod_accountOwner', $account_owner );
    }

    /**
     * Save delivery option
     *
     * @since 1.0.0
     */
    private function save_delivery()
    {
        if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) ) ) {
            $map_to_delivery = map_deep( wp_unslash( isset( $_POST['mapToDelivery'] ) ?
                $_POST['mapToDelivery'] : [] ), 'sanitize_text_field' );
            $result_array    = array();
            $fail            = false;
            if ( ! empty( $map_to_delivery ) ) {
                foreach ( $map_to_delivery as $type => $options ) {
                    foreach ( $options as $option ) {
                        if ( isset( $result_array[ sanitize_text_field( $option ) ] ) ) {
                            $fail = true;
                            break 2;
                        }
                        $result_array[ $option ] = sanitize_text_field( $type );
                    }
                }
            }
            if ( $fail ) {
                $this->errors[] = esc_html__( 'Every delivery option can have just one map attached.', 'furgonetka' );
                return;
            }
            update_option( $this->plugin_name . '_deliveryToType', $result_array );
        }
    }

    /**
     * Reset credentials
     *
     * @since 1.0.0
     */
    private function reset_credentials()
    {
        delete_option( $this->plugin_name . '_client_ID' );
        delete_option( $this->plugin_name . '_client_secret' );
        delete_option( $this->plugin_name . '_access_token' );
        delete_option( $this->plugin_name . '_refresh_token' );
        delete_option( $this->plugin_name . '_expires_date' );
        delete_option( $this->plugin_name . '_test_mode' );
        delete_option( $this->plugin_name . '_email' );
        delete_option( $this->plugin_name . '_source_id' );
        delete_option( $this->plugin_name . '_integration_uuid' );

        delete_option( $this->plugin_name . '_key_consumer_key' );
        delete_option( $this->plugin_name . '_key_consumer_secret' );

        $this->messages[] = __( 'Account reseted', 'furgonetka' );
    }

    /**
     * Get product html selector to put portmonetka button in
     *
     * @since 1.2.2
     */
    public static function get_portmonetka_product_selector()
    {
        return self::get_plugin_option( 'portmonetka_product_selector' );
    }

    /**
     * Get cart html selector to put portmonetka button in
     *
     * @since 1.2.2
     */
    public static function get_portmonetka_cart_selector()
    {
        return self::get_plugin_option( 'portmonetka_cart_selector' );
    }

    /**
     * Get cart html selector to put portmonetka button in
     *
     * @since 1.2.2
     */
    public static function get_portmonetka_minicart_selector()
    {
        return self::get_plugin_option( 'portmonetka_minicart_selector' );
    }

    /**
     * Save advanced settings
     *
     * @since 1.2.2
     */
    private function save_advanced_settings()
    {
        $product_selector  = '';
        $cart_selector     = '';
        $minicart_selector = '';

        if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) ) ) {
            $product_selector  = isset( $_POST['portmonetka_product_selector'] ) ?
                sanitize_text_field( wp_unslash( $_POST['portmonetka_product_selector'] ) ) : '';
            $cart_selector     = isset( $_POST['portmonetka_cart_selector'] ) ?
                sanitize_text_field( wp_unslash( $_POST['portmonetka_cart_selector'] ) ) : '';
            $minicart_selector = isset( $_POST['portmonetka_minicart_selector'] ) ?
                sanitize_text_field( wp_unslash( $_POST['portmonetka_minicart_selector'] ) ) : '';
        }

        update_option( $this->plugin_name . '_portmonetka_product_selector', $product_selector );
        update_option( $this->plugin_name . '_portmonetka_cart_selector', $cart_selector );
        update_option( $this->plugin_name . '_portmonetka_minicart_selector', $minicart_selector );
    }

    /**
     * Create Integration OAuth Application
     *
     * @since 1.0.0
     */
    private function create_integration()
    {
        $ck = '';
        $cs = '';

        if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) ) ) {
            $ck = isset( $_POST['key_consumer_key'] ) ?
                sanitize_text_field( wp_unslash( $_POST['key_consumer_key'] ) ) : '';
            $cs = isset( $_POST['key_consumer_secret'] ) ?
                sanitize_text_field( wp_unslash( $_POST['key_consumer_secret'] ) ) : '';
        }

        $url = self::get_test_mode() ? self::TEST_API_SOAP_URL : self::API_SOAP_URL;

        $client = new \SoapClient(
            $url,
            array(
                'trace'      => true,
                'cache_wsdl' => false,
                'user_agent' => self::get_request_user_agent(),
            )
        );

        $api_data = array(
            'integration_type' => 'woocommerce',
            'url'              => self::get_redirect_uri(),
            'data_1'           => get_home_url(),
            'data_2'           => $ck,
            'data_3'           => $cs,
        );

        $result = $client->createIntegrationOAuthApplication(
            array(
                'data' => $api_data,
            )
        )->createIntegrationOAuthApplicationResult;

        if ( ! $result->client_id ) {
            $this->errors[] = $result->errors->item->message;
        } else {
            $test_mode = self::get_test_mode() ? true : false;
            update_option( $this->plugin_name . '_client_ID', $result->client_id );
            update_option( $this->plugin_name . '_client_secret', $result->client_secret );

            /**
             * Save wp access api data in db
             */
            update_option( $this->plugin_name . '_key_consumer_key', $ck );
            update_option( $this->plugin_name . '_key_consumer_secret', $cs );

            /**
             * Woocommerce api keys
             */
            update_option( $this->plugin_name . '_woo_ck', $ck );
            update_option( $this->plugin_name . '_woo_cs', password_hash( $cs, PASSWORD_DEFAULT ) );

            update_option( $this->plugin_name . '_test_mode', $test_mode );
            $this->messages[] = esc_html__( 'Integration created. Please add credentials.', 'furgonetka' );

            $url   = self::get_test_mode() ? self::TEST_API_OAUTH_URL : self::API_OAUTH_URL;
            $query = http_build_query(
                array(
                    'client_id'    => $result->client_id,
                    'redirect_uri' => self::get_redirect_uri(),
                    'state'        => self::get_oauth_state(),
                )
            );
            $url  .= '/authorize?response_type=code&' . $query;
            header( 'Location: ' . $url );
            die();
        }
    }

    public static function update_plugin_version( $version )
    {
        if ( ! self::get_rest_customer_key() || ! self::get_rest_customer_secret() ) {
            return;
        }

        $token = self::get_access_token();

        if ( ! $token ) {
            return;
        }

        $integration_uuid = self::get_integration_uuid();
        $source_id        = self::get_source_id();

        if ( ! $integration_uuid && ! $source_id ) {
            return;
        }

        $url         = self::get_test_mode() ? 'https://api.sandbox.furgonetka.pl' : 'https://api.furgonetka.pl';
        $body_params = array( 'version' => $version );

        if ( $integration_uuid ) {
            $url .= '/e-commerce/integrations/' . $integration_uuid . '/plugin';
        } elseif ( $source_id ) {
            $url        .= '/e-commerce/integrations/plugin';
            $body_params = array_merge( $body_params, array( 'sourceId' => $source_id ) );
        }

        $args = array(
            'headers'    => array(
                'Accept'        => '*/*',
                'Cache-Control' => 'no-cache',
                'content-type'  => 'application/json',
                'Authorization' => 'Bearer ' . $token,
            ),
            'method'     => 'PATCH',
            'body'       => json_encode( $body_params ),
            'user-agent' => self::get_request_user_agent(),
        );

        wp_remote_request( $url, $args );
    }

    /**
     * Get access tocken by code
     *
     * @since 1.0.0.
     * @param  mixed $code          - code.
     * @param  mixed $client_id     - client id.  - id of the client.
     * @param  mixed $client_secret - client secret.  - client secret.
     * @param  mixed $test_mode     - set test mode. - set if in test mode.
     * @throws Exception - curl errors.
     */
    private function grant_code_access( $code, $client_id, $client_secret, $test_mode = null )
    {
        //phpcs:ignore
        $auth = base64_encode( $client_id . ':' . $client_secret );
        $url  = self::get_test_mode() ? self::TEST_API_OAUTH_URL : self::API_OAUTH_URL;
        $url .= '/token';

        $args     = array(
            'headers'    => array(
                'Authorization' => 'Basic ' . $auth,
                'content-type'  => 'application/x-www-form-urlencoded',
            ),
            'method'     => 'POST',
            'body'       => http_build_query(
                array(
                    'grant_type'   => 'authorization_code',
                    'code'         => $code,
                    'redirect_uri' => self::get_redirect_uri(),
                )
            ),
            'user-agent' => self::get_request_user_agent(),
        );
        $response = wp_remote_request( $url, $args );

        if ( is_wp_error( $response ) ) {
            throw new Exception( 'Curl error: ' . $response->get_error_message() );
        } else {
            $http_status   = wp_remote_retrieve_response_code( $response );
            $server_output = trim( wp_remote_retrieve_body( $response ) );
            if ( $http_status >= 400 ) {
                $error = json_decode( $server_output );
                throw new Exception( 'HTTP STATUS: ' . $http_status . ' - Message: ' . $error->message );
            } else {
                if ( 200 === $http_status ) {
                    $response = json_decode( $server_output );
                    if ( isset( $response->access_token ) ) {
                        update_option( $this->plugin_name . '_access_token', $response->access_token );
                    }
                    if ( isset( $response->refresh_token ) ) {
                        update_option( $this->plugin_name . '_refresh_token', $response->refresh_token );
                    }
                    if ( isset( $response->expires_in ) ) {
                        $expires_date = strtotime( 'now' ) + $response->expires_in;
                        update_option( $this->plugin_name . '_expires_date', $expires_date );
                    }
                    $test_mode = $test_mode ? true : false;
                    update_option( $this->plugin_name . '_test_mode', $test_mode );
                } else {
                    throw new Exception( 'BAD HTTP STATUS: ' . $http_status );
                }
            }
        }
    }

    /**
     * Add integration source
     *
     * @since 1.0.0
     *
     * @param  mixed $access_token        - acces token.
     * @param  mixed $key_consumer_key    - consumer key.
     * @param  mixed $key_consumer_secret - secret key.
     * @throws \Exception - integration source error.
     */
    private function add_integration_source( $access_token, $key_consumer_key, $key_consumer_secret )
    {
        $url    = self::get_test_mode() ? self::TEST_API_SOAP_URL : self::API_SOAP_URL;
        $client = new \SoapClient(
            $url,
            array(
                'trace'      => true,
                'cache_wsdl' => false,
                'user_agent' => self::get_request_user_agent(),
            )
        );
        $auth   = array(
            'access_token' => $access_token,
        );

        $api_data = array(
            'auth'             => $auth,
            'integration_type' => 'woocommerce',
            'module_version'   => $this->version,
            'data_1'           => get_home_url(),
            'data_2'           => $key_consumer_key,
            'data_3'           => $key_consumer_secret,
        );

        delete_option( $this->plugin_name . '_key_consumer_key' );
        delete_option( $this->plugin_name . '_key_consumer_secret' );

        $result = $client->addIntegrationSource(
            array(

                'data' => $api_data,

            )
        )->addIntegrationSourceResult;

        if ( $result->errors ) {
            if ( ! empty( $result->errors ) ) {
                throw new \Exception( $result->errors->item->message . ' : ' . $result->errors->item->field );
            }
            throw new \Exception( __( 'Add integration source problem', 'furgonetka' ) );
        } else {
            return $result;
        }
    }

    /**
     * Refresh furgonetka token
     *
     * @since 1.0.0
     */
    public function furgonetka_refresh_token()
    {
        /** Break if expiras date > 7 days */
        if ( get_option( $this->plugin_name . '_expires_date' ) > strtotime( '+7 day' ) ) {
            return;
        }

        $test_mode     = get_option( $this->plugin_name . '_test_mode' );
        $client_id     = get_option( $this->plugin_name . '_client_ID' );
        $client_secret = get_option( $this->plugin_name . '_client_secret' );
        $refresh_token = get_option( $this->plugin_name . '_refresh_token' );

        try {
            $this->refresh_token( $client_id, $client_secret, $test_mode, $refresh_token );
            $this->messages[] = __( 'Access granted', 'furgonetka' );
        } catch ( Exception $e ) {
            $this->errors[] = $e->getMessage();
        }
    }

    /**
     * Refresh user tocken
     *
     * @since 1.0.0
     *
     * @param mixed $client_id     - client id.
     * @param mixed $client_secret - client secret.
     * @param mixed $test_mode     - set test mode.
     * @param mixed $refresh_token - refresh token.
     *
     * @throws Exception - http status.
     */
    private function refresh_token( $client_id, $client_secret, $test_mode, $refresh_token )
    {
        //phpcs:ignore
        $auth = base64_encode( $client_id . ':' . $client_secret );
        $url  = $test_mode ? self::TEST_API_OAUTH_URL : self::API_OAUTH_URL;
        $url .= '/token';

        $args = array(
            'headers'    => array(
                'Authorization' => 'Basic ' . $auth,
                'content-type'  => 'application/x-www-form-urlencoded',
            ),
            'method'     => 'POST',
            'body'       => http_build_query(
                array(
                    'grant_type'    => 'refresh_token',
                    'refresh_token' => $refresh_token,
                    'redirect_uri'  => self::get_redirect_uri(),
                )
            ),
            'user-agent' => self::get_request_user_agent(),
        );

        $response = wp_remote_request( $url, $args );

        if ( is_wp_error( $response ) ) {
            throw new Exception( 'Curl error: ' . $response->get_error_message() );
        } else {
            $http_status   = wp_remote_retrieve_response_code( $response );
            $server_output = trim( wp_remote_retrieve_body( $response ) );

            if ( $http_status >= 400 ) {
                $error = json_decode( $server_output );
                throw new Exception( 'HTTP STATUS: ' . $http_status . ' - Message: ' . $error->message );
            } else {
                if ( 200 === $http_status ) {
                    $response = json_decode( $server_output );
                    if ( isset( $response->access_token ) ) {
                        update_option( $this->plugin_name . '_access_token', $response->access_token );
                    }
                    if ( isset( $response->refresh_token ) ) {
                        update_option( $this->plugin_name . '_refresh_token', $response->refresh_token );
                    }
                    if ( isset( $response->expires_in ) ) {
                        $expires_date = strtotime( 'now' ) + $response->expires_in;
                        update_option( $this->plugin_name . '_expires_date', $expires_date );
                    }
                } else {
                    throw new Exception( 'BAD HTTP STATUS: ' . $http_status );
                }
            }
        }
    }

    /**
     * Get user balance from API
     *
     * @param mixed $access_token - api token.
     *
     * @throws \Exception - Get balance problem.
     *
     * @return strting
     */
    public static function get_balance( $access_token )
    {
        $url    = self::get_test_mode() ? self::TEST_API_SOAP_URL : self::API_SOAP_URL;
        $client = new \SoapClient(
            $url,
            array(
                'trace'      => true,
                'cache_wsdl' => false,
                'user_agent' => self::get_request_user_agent(),
            )
        );
        $auth   = array(
            'access_token' => $access_token,
        );

        $balance = $client->getBalance(
            array(

                'data' => array(
                    'auth' => $auth,
                ),

            )
        )->getBalanceResult;
        if ( ! isset( $balance->balance ) ) {
            if ( ! empty( $balance->errors ) ) {
                throw new \Exception( $balance->errors->item->message );
            }
            throw new \Exception( __( 'Balance GET problem', 'furgonetka' ) );
        } else {
            return $balance->balance;
        }
    }

    /**
     * Get Package details from API
     *
     * @param mixed $access_token - access token.
     * @param mixed $reference    - reference number.
     *
     * @since 1.0.0
     */
    public static function get_package_details( $access_token, $reference )
    {
        $url    = self::get_test_mode() ? self::TEST_API_SOAP_URL : self::API_SOAP_URL;
        $client = new \SoapClient(
            $url,
            array(
                'trace'      => true,
                'cache_wsdl' => false,
                'user_agent' => self::get_request_user_agent(),
            )
        );
        $auth   = array(
            'access_token' => $access_token,
        );

        $result = $client->get_package_details(
            array(

                'data' => array(
                    'auth'                     => $auth,
                    'partner_reference_number' => $reference,
                ),

            )
        )->get_package_detailsResult;

        return $result;
    }

    /**
     * Get package form url from API
     *
     * @param mixed $access_token - access token.
     * @param mixed $order_id     - order ID.
     *
     * @throws \Exception - url package error.
     *
     * @since 1.0.0
     */
    public static function get_package_form_url( $access_token, $order_id )
    {
        $order_data = new WC_Order( $order_id );
        $parcels    = array();
        $services   = array();

        if ( $order_data->get_payment_method() === 'cod' ) {
            $services['cod'] = array(
                'amount' => $order_data->get_total(),
                'iban'   => self::get_iban(),
                'name'   => self::get_account_owner(),
            );
        }
        foreach ( $order_data->get_items() as $item ) {
            $product_name = $item['name'];
            $parcels[]    = array(
                'description' => $product_name,
                'value'       => $order_data->get_total(),
            );
        }
        $sender   = self::get_sender();
        $receiver = self::getr_receiver( $order_data );

        $url    = self::get_test_mode() ? self::TEST_API_SOAP_URL : self::API_SOAP_URL;
        $client = new \SoapClient(
            $url,
            array(
                'trace'      => true,
                'cache_wsdl' => false,
                'user_agent' => self::get_request_user_agent(),
            )
        );
        $auth   = array(
            'access_token' => $access_token,
        );

        $reference = $order_data->get_order_number();

        if ( empty( $reference ) ) {
            $reference = $order_id;
        }

        $result = $client->getPackageFormUrl(
            array(

                'data' => array(
                    'auth'                     => $auth,
                    'sender'                   => $sender,
                    'receiver'                 => $receiver,
                    'services'                 => $services,
                    'service'                  => self::get_service( $order_data ),
                    'type'                     => 'package',
                    'partner_reference_number' => $reference,
                    'user_reference_number'    => $reference,
                    'parcels'                  => $parcels,
                    'sale_source_id'           => self::get_source_id(),
                ),

            )
        )->getPackageFormUrlResult;

        if ( ! $result->url ) {
            if ( ! empty( $result->errors ) ) {
                throw new \Exception( $result->errors->item->field . ': ' . $result->errors->item->message );
            }
            throw new \Exception( __( 'Get package Form URL problem.', 'furgonetka' ) );
        } else {
            return $result->url;
        }
    }

    /**
     * Get sender data from options
     *
     * @since 1.0.0
     */
    public static function get_sender()
    {
        $data = array(
            'name'         => self::get_name(),
            'email'        => self::get_sender_email(),
            'company'      => self::get_company_name(),
            'street'       => self::get_street(),
            'postcode'     => self::get_post_code(),
            'city'         => self::get_city(),
            'country_code' => '',
            'county'       => '',
            'phone'        => self::get_telephone(),
            'point'        => '',
        );
        return $data;
    }

    /**
     * Get receiver from order
     *
     * @param mixed $order_data - order data.
     *
     * @since 1.0.0
     */
    public static function getr_receiver( $order_data )
    {
        if ( $order_data ) {
            $point        = get_post_meta( $order_data->get_id(), '_furgonetkaPoint', true );
            $name         = ! empty( $order_data->get_shipping_first_name() ) ?
                $order_data->get_shipping_first_name() . ' ' . $order_data->get_shipping_last_name()
                : $order_data->get_billing_first_name() . ' ' . $order_data->get_billing_last_name();
            $email        = $order_data->get_billing_email();
            $company      = ! empty( $order_data->get_shipping_company() ) ?
                $order_data->get_shipping_company() : $order_data->get_billing_company();
            $street       = ! empty( $order_data->get_shipping_address_1() ) ?
                $order_data->get_shipping_address_1() . ' ' . $order_data->get_shipping_address_2()
                : $order_data->get_billing_address_1() . ' ' . $order_data->get_billing_address_2();
            $post_code    = ! empty( $order_data->get_shipping_postcode() ) ?
                $order_data->get_shipping_postcode() : $order_data->get_billing_postcode();
            $city         = ! empty( $order_data->get_shipping_city() ) ?
                $order_data->get_shipping_city() : $order_data->get_billing_city();
            $country_code = ! empty( $order_data->get_shipping_country() ) ?
                $order_data->get_shipping_country() : $order_data->get_billing_country();
            $phone        = $order_data->get_billing_phone();

            $data = array(
                'name'              => $name,
                'email'             => $email,
                'company'           => $company,
                'street'            => $street,
                'postcode'          => $post_code,
                'city'              => $city,
                'country_code'      => $country_code,
                'county'            => '',
                'phone'             => $phone,
                'point'             => $point,
                'alternative_point' => '',
            );
        } else {
            $data = array(
                'name'              => '',
                'email'             => '',
                'company'           => '',
                'street'            => '',
                'postcode'          => '',
                'city'              => '',
                'country_code'      => '',
                'county'            => '',
                'phone'             => '',
                'point'             => '',
                'alternative_point' => '',
            );
        }
        return $data;
    }

    /**
     * Get furgonetka service name from order
     *
     * @param mixed $order_data - order data.
     *
     * @since 1.0.0.
     */
    public static function get_service( $order_data )
    {
        // Dhl, dpd, fedex, ups, inpost, inpostkurier, poczta, kex, ruch, xpress, gls.
        $shipping_methods = $order_data->get_shipping_methods();
        $shipping_name    = '';
        foreach ( $shipping_methods as $shipping_method ) {
            $data = $shipping_method->get_data();

            $shipping_name = $data['method_id'] . ':' . $data['instance_id'];
            if ( 'flexible_shipping' === $data['method_id'] ) {
                if (
                    isset( $shipping_method['item_meta'] )
                    && isset( $shipping_method['item_meta']['_fs_method'] )
                ) {
                    $fs_method     = $shipping_method['item_meta']['_fs_method'];
                    $shipping_name = $fs_method['id_for_shipping'];
                }
            }
        }

        $delivery_to_type = get_option( FURGONETKA_PLUGIN_NAME . '_deliveryToType' );
        $service          = null;

        if ( isset( $delivery_to_type[ $shipping_name ] ) ) {
            $service = $delivery_to_type[ $shipping_name ];
        } elseif ( $order_data ) {
            $service = $order_data->get_meta( '_furgonetkaService' );
        }

        if ( ! empty( $service ) ) {
            $type = '';
            switch ( $service ) {
                case 'inpost':
                    $type = 'inpost';
                    break;
                case 'poczta':
                    $type = 'poczta';
                    break;
                case 'kiosk':
                    $type = 'ruch';
                    break;
                case 'uap':
                    $type = 'ups';
                    break;
                case 'dpd':
                    $type = 'dpd';
                    break;
                case 'dhl':
                    $type = 'dhl';
                    break;
                case 'fedex':
                    $type = 'fedex';
                    break;
                case 'gls':
                    $type = 'gls';
                    break;
                case 'orlen':
                    $type = 'orlen';
                    break;
                default:
                    break;
            }
            return $type;
        }
        if ( ! $order_data ) {
            return;
        }

        return '';
    }

    /**
     * Get email
     *
     * @since 1.0.0.
     */
    public static function get_email()
    {
        return self::check_nonce_and_get_data( 'email', '_email' );
    }

    /**
     * Get client ID
     *
     * @since 1.0.0.
     */
    public static function get_cient_id()
    {
        return self::check_nonce_and_get_data( 'clientID', '_client_ID' );
    }

    /**
     * Get client balance
     *
     * @since 1.0.0.
     */
    public static function get_client_balance()
    {
        $access_token = get_option( FURGONETKA_PLUGIN_NAME . '_access_token' );

        try {
            $balance = self::get_balance( $access_token );
            return $balance;
        } catch ( Exception $e ) {
            return $e->getMessage();
        }
        return 0;
    }

    /**
     * Get client service
     *
     * @since 1.0.0.
     */
    public static function get_client_secret()
    {
        return self::check_nonce_and_get_data( 'clientSecret', '_client_secret' );
    }

    /**
     * Get test mode
     *
     * @since 1.0.0.
     */
    public static function get_test_mode()
    {
        $action    = (
                isset( $_POST['_wpnonce'], $_POST['furgonetkaAction'] )
                && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) )
        ) ? sanitize_text_field( wp_unslash( $_POST['furgonetkaAction'] ) ) : '';
        $test_mode = (
            isset( $_POST['_wpnonce'], $_POST['isTestMode'] )
            && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) )
        ) ? sanitize_text_field( wp_unslash( $_POST['isTestMode'] ) ) === '1' : false;
        return (
            isset( $action )
            && ( 'create_integration' === $action )
        ) ? ( $test_mode ) : get_option( FURGONETKA_PLUGIN_NAME . '_test_mode' );
    }

    /**
     * Check nonce and get data from options table
     *
     * @param  string $post_field_name - field name from POST table.
     * @param  string $option_name     - option name in DB.
     * @return mixed
     */
    public static function check_nonce_and_get_data( $post_field_name, $option_name )
    {
        return (
            isset( $_POST['_wpnonce'], $_POST[ $post_field_name ] )
            && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) )
        ) ? sanitize_text_field( wp_unslash( $_POST[ $post_field_name ] ) )
            : get_option( FURGONETKA_PLUGIN_NAME . $option_name );
    }

    /**
     * Get Name
     *
     * @since 1.0.0.
     */
    public static function get_name()
    {
        return self::check_nonce_and_get_data( 'name', '_sender_name' );
    }

    /**
     * Get compoany name
     *
     * @since 1.0.0.
     */
    public static function get_company_name()
    {
        return self::check_nonce_and_get_data( 'companyName', '_sender_companyName' );
    }

    /**
     * Get post code
     *
     * @since 1.0.0.
     */
    public static function get_post_code()
    {
        return self::check_nonce_and_get_data( 'postCode', '_sender_postCode' );
    }

    /**
     * Get city
     *
     * @since 1.0.0.
     */
    public static function get_city()
    {
        return self::check_nonce_and_get_data( 'city', '_sender_city' );
    }

    /**
     * Get street
     *
     * @since 1.0.0.
     */
    public static function get_street()
    {
        return self::check_nonce_and_get_data( 'street', '_sender_street' );
    }

    /**
     * Get sender email
     *
     * @since 1.0.0.
     */
    public static function get_sender_email()
    {
        return self::check_nonce_and_get_data( 'email', '_sender_email' );
    }

    /**
     * Get sender phone
     *
     * @since 1.0.0.
     */
    public static function get_telephone()
    {
        return self::check_nonce_and_get_data( 'telephone', '_sender_telephone' );
    }

    /**
     * Get iban
     *
     * @since 1.0.0.
     */
    public static function get_iban()
    {
        return self::check_nonce_and_get_data( 'iban', '_cod_iban' );
    }

    /**
     * Get source id
     *
     * @since 1.0.0.
     */
    public static function get_source_id()
    {
        return get_option( FURGONETKA_PLUGIN_NAME . '_source_id' );
    }

    /**
     * Get access token
     *
     * @since    1.2.0.
     */
    public static function get_access_token()
    {
        return get_option( FURGONETKA_PLUGIN_NAME . '_access_token' );
    }

    /**
     * Get integration uuid
     *
     * @since    1.2.0.
     */
    public static function get_integration_uuid()
    {
        return get_option( FURGONETKA_PLUGIN_NAME . '_integration_uuid' );
    }

    /**
     * Get account owner
     *
     * @since 1.0.0.
     */
    public static function get_account_owner()
    {
        return self::check_nonce_and_get_data( 'accountOwner', '_cod_accountOwner' );
    }

    /**
     * Get redirect Uri
     *
     * @since 1.0.0.
     */
    public static function get_redirect_uri()
    {
        return admin_url( 'admin.php?page=furgonetka&tab=furgonetka&action=oAuthComplete' );
    }

    /**
     * Get Oauth state
     *
     * @since 1.0.0.
     */
    public static function get_oauth_state()
    {
        return wp_create_nonce( 'furgonetka_csrf' );
    }

    /**
     * Generate multiselect with delivery options
     *
     * @param mixed $type             - field type.
     * @param mixed $delivery_to_type - delivery type.
     *
     * @since 1.0.0.
     */
    public static function map_attach_to( $type, $delivery_to_type )
    {
        if ( ! $type ) {
            return;
        }
        $options = '';

        $zones = WC_Shipping_Zones::get_zones();

        /**
         * Add "0" zone (that contains shipping methods without assigned real zone)
         */
        $fallback_zone = WC_Shipping_Zones::get_zone( 0 );

        if ( $fallback_zone ) {
            /**
             * Get zone data & assigned shipping methods
             */
            $data                     = $fallback_zone->get_data();
            $data['shipping_methods'] = $fallback_zone->get_shipping_methods( false, 'admin' );

            /**
             * Push zone to the array
             */
            $zones[ $fallback_zone->get_id() ] = $data;
        }
        ?>
        <select multiple size="6" name="mapToDelivery[<?php echo esc_html( $type ); ?>][]">
        <?php
        $supported_shipping_methods_ids = array( 'flat_rate', 'flexible_shipping_single', 'free_shipping' );
        foreach ( $zones as $zone_item ) {
            $shipping_methods = $zone_item['shipping_methods'];
            foreach ( $shipping_methods as $shipping_method ) {
                if ( ! in_array( $shipping_method->id, $supported_shipping_methods_ids, true ) ) {
                    continue;
                }
                $instance = $shipping_method->id . ':' . $shipping_method->instance_id;
                ?>
                    <option
                            value="<?php echo esc_html( $instance ); ?>"
                            <?php echo self::check_selected( $type, $instance, $delivery_to_type ) ?
                                'selected' : ''; ?>
                    >
                        <?php echo esc_html( $zone_item['zone_name'] ) . ':' . esc_html( $shipping_method->title ); ?>
                    </option>
                <?php
            }
        }
        ?>
        </select>
        <?php
    }

    /**
     * Admin print messages
     *
     * @param mixed $messages - message.
     * @param mixed $type     - message type.
     *
     * @since 1.0.0.
     */
    public static function print_messages( $messages, $type )
    {
        if ( ! $messages ) {
            return;
        }

        if ( ! $type ) {
            $type = 'message';
        }

        if ( ! is_array( $messages ) ) {
            return;
        }

        foreach ( $messages as $message ) {
            echo sprintf(
                '<div id="message" class="updated woocommerce-%1$s inline">
                        <p>%2$s</p>
                    </div>',
                esc_html( $type ),
                esc_html( $message )
            );
        }
    }

    /**
     * Check if map is attach to delivery option
     *
     * @param mixed $type             - type.
     * @param mixed $instance         - instance.
     * @param mixed $delivery_to_type - delivery type.
     *
     * @since 1.0.0
     */
    public static function check_selected( $type, $instance, $delivery_to_type )
    {

        if (
            isset( $_POST['_wpnonce'], $_POST['mapToDelivery'] )
            && wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ) )
        ) {
            if (
                isset( $_POST['mapToDelivery'][ $type ] )
                && in_array( $instance, $_POST['mapToDelivery'][ $type ], true )
            ) {
                return true;
            }
        } else {
            if ( isset( $delivery_to_type[ $instance ] ) && $delivery_to_type[ $instance ] === $type ) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get woocommerce version
     *
     * @since 1.0.0
     */
    private static function get_wc_version()
    {
        if ( function_exists( 'WC' ) ) {
            return WC()->version;
        }
    }

    /**
     * Check if account is active and enabled
     *
     * @since 1.0.0
     */
    public static function is_account_active()
    {
        if ( ! get_option( FURGONETKA_PLUGIN_NAME . '_expires_date' ) ) {
            return false;
        }
        if ( get_option( FURGONETKA_PLUGIN_NAME . '_expires_date' ) < strtotime( 'now' ) ) {
            return false;
        }

        return true;
    }

    /**
     * Get user agent
     *
     * @return string
     */
    private static function get_request_user_agent()
    {
        return 'woocommerce_' . self::get_wc_version() . '_plugin_' . FURGONETKA_VERSION;
    }

    /**
     * Get rest customer key
     *
     * @return string
     */
    public static function get_rest_customer_key()
    {
        return get_option( FURGONETKA_PLUGIN_NAME . '_woo_ck' );
    }

    /**
     * Get rest customer secret
     *
     * @return string
     */
    public static function get_rest_customer_secret()
    {
        return get_option( FURGONETKA_PLUGIN_NAME . '_woo_cs' );
    }
}
