<?php

/**
 * The file that defines class for registering metaboxes
 *
 * @link  https://furgonetka.pl
 * @since 1.0.0
 *
 * @package    Furgonetka
 * @subpackage Furgonetka/includes
 */

/**
 *  Class furgonetka_admin_metaboxes - Manage custom metaboxes in single orders and custom columns in orders page
 *
 * @since      1.0.0
 * @package    Furgonetka
 * @subpackage Furgonetka/includes
 * @author     Furgonetka.pl <woocommerce@furgonetka.pl>
 */
class furgonetka_admin_metaboxes
{
    /**
     * View
     *
     * @var \Furgonetka_Admin_Metaboxes_View
     */
    private $view;

    /**
     * Model
     *
     * @var \Furgonetka_Update_Order_Model
     */
    private $model;

    /**
     * Admin class
     *
     * @var \Furgonetka_Admin
     */
    private $furgonetka_admin;

    /**
     * Include view and model
     *
     * @param Furgonetka_Admin $furgonetka_admin - admin class.
     */
    public function __construct( Furgonetka_Admin $furgonetka_admin )
    {
        $this->furgonetka_admin = $furgonetka_admin;

        require_once FURGONETKA_PLUGIN_DIR . 'includes/view/class-furgonetka-admin-metaboxes-view.php';
        $this->view = new Furgonetka_Admin_Metaboxes_View();

        // require_once FURGONETKA_PLUGIN_DIR . 'includes/rest_api/models/class-furgonetka-returns-model.php'.
        require_once FURGONETKA_PLUGIN_DIR . 'includes/rest_api/models/class-furgonetka-update-order-model.php';
        $this->model = new Furgonetka_Update_Order_Model();
    }

    /**
     * Get package link in meta box
     *
     *  1.0.0
     */
    public function other_package_link_callback()
    {
        global $post;
        $this->other_package_link( $post );

        $point      = esc_html( get_post_meta( $post->ID, '_furgonetkaPoint', true ) );
        $point_name = esc_html( get_post_meta( $post->ID, '_furgonetkaPointName', true ) );
        $service    = esc_html( get_post_meta( $post->ID, '_furgonetkaService', true ) );

        if ( $service ) {
            echo esc_html( $this->get_name_of_service( $service ) ) . '<br/>' . ( esc_html( $point_name ) ? esc_html( $point_name ) : esc_html( $point ) );
        }
    }

    /**
     * Get name of service
     *
     * @param string $service - service slug.
     *
     * @return string
     */
    public function get_name_of_service( string $service )
    {
        $result = '';
        switch ( $service ) {
            case 'inpost':
                $result = 'Inpost paczkomat';
                break;
            case 'poczta':
                $result = 'Poczta';
                break;
            case 'kiosk':
                $result = 'Paczka w ruchu';
                break;
            case 'uap':
                $result = 'UPS Access Point';
                break;
            case 'dpd':
                $result = 'DPD Pickup';
                break;
            case 'dhl':
                $result = 'DHL Parcel';
                break;
            case 'fedex':
                $result = 'FedEx Punkt';
                break;
            case 'gls':
                $result = 'GLS Szybka Paczka';
                break;
            case 'orlen':
                $result = 'ORLEN Paczka';
                break;
            default:
                break;
        }
        return $result;
    }

    /**
     * Add furgonetka meta boxes for order
     *
     * @since 1.0.0
     */
    public function furgonetka_meta_boxes()
    {
        add_meta_box( $this->furgonetka_admin->get_plugin_name() . '_delivery', __( 'Furgonetka.pl', 'furgonetka' ), array( $this, 'render_metabox_for_product_page' ), 'shop_order', 'side', 'core' );
    }

    /**
     * Render html for metaboxes
     */
    public function render_metabox_for_product_page(): void
    {
        global $post;
        $this->other_package_link_callback();
        $this->package_information( $post );
    }

    /**
     * Render html for package info
     *
     * @param mixed $post - post data.
     */
    public function package_information( $post )
    {
        $this->view->render_packages_tracking_info( $this->render_packages_numbers( $post ) );
    }

    /**
     * Renders link to furgonetka iframe
     *
     * @param mixed $post - post data.
     */
    public function other_package_link( $post )
    {
        $this->view->render_package_link(
            get_admin_url( null, 'admin.php?page=' . $this->furgonetka_admin->get_plugin_name() . '&idOrder=' . $post->ID ),
            plugin_dir_url( __DIR__ ) . 'admin/img/furgonetka-kurirer.svg'
        );
    }

    /**
     * Adding new columns to orders page
     *
     * @param array $columns - columns array.
     *
     * @return array
     */
    public function extra_order_column( array $columns ): array
    {
        $columns[ $this->furgonetka_admin->get_plugin_name() ] = __( 'Furgonetka.pl', 'furgonetka' );
        $columns['package_number']                             = __( 'Packages tracking numbers', 'furgonetka' );
        return $columns;
    }

    /**
     * Render package number.
     *
     * @param mixed $post - post data.
     *
     * @return string|void
     */
    public function render_packages_numbers( $post )
    {
        $package_numbers = $this->model
            ->set_order_id( $post->ID )
            ->get_tracking_info();
        if ( empty( $package_numbers ) ) {
            return;
        }
        return $package_numbers;
    }

    /**
     * Get package numbers
     *
     * @param mixed $post - post data.
     *
     * @return array|void
     */
    public function get_packages_numbers( $post )
    {
        $package_numbers = $this->model
            ->set_order_id( $post->ID )
            ->get_tracking_info();
        if ( empty( $package_numbers ) ) {
            return array();
        }
        return $package_numbers;
    }

    /**
     * Adding content to new columns in orders page
     *
     * @param string $column - column name.
     *
     * @return void
     */
    public function extra_order_column_content( string $column ): void
    {
        global $post;

        switch ( $column ) {
            case $this->furgonetka_admin->get_plugin_name():
                $this->other_package_link( $post );
                break;
            case 'package_number':
                $package_numbers = $this->get_packages_numbers( $post );
                foreach ( $package_numbers as $package_number => $package_data ) {

                    if ( Furgonetka_Admin::get_test_mode() ) {
						echo '<a style="text-decoration : underline" target="_blank" href="https://sandbox.furgonetka.pl/zlokalizuj/' . esc_html( $package_number ) . '">' . esc_html( $package_number ) . '</a><br>';
                    } else {
                        echo '<a style="text-decoration : underline" target="_blank" href="https://furgonetka.pl/zlokalizuj/' . esc_html( $package_number ) . '">' . esc_html( $package_number ) . '</a><br>';
                    }
                }
                break;
        }
    }
}
